@@ -3,18 +3,10 @@
 
 #########################
 
-# change 'tests => 1' to 'tests => last_test_to_print';
-
-use Test;
-BEGIN { plan tests => 1 };
+use Test::More tests => 8;
 use Data::Pivot;
-ok(1); # If we made it this far, we're ok.
-
-#########################
-
-# Insert your test code below, the Test module is use()ed here so read
-# its man page ( perldoc Test ) for help writing this test script.
 
+###############################################################################
 my @table = ( [ 'a', 'b', 'c', 1, 10 ],
               [ 'a', 'b', 'c', 2, 20 ],
               [ 'a', 'b', 'c', 3, 30 ],
@@ -27,8 +19,12 @@ my @fix_cols = ( 0, 1, 2 );
 my $pivot_col = 3;
 my @sum_cols = ( 4 );
 
-print "Before:\n";
-out(\@headings, \@table);
+my @expect_table = (
+    [ 'a', 'b', 'c', '10.00', '20.00', '30.00' ],
+    [ 'x', 'y', 'z', ' 1.00', ' 2.00', ' 3.00' ],
+    );
+my @expect_headings = ('A', 'B', 'C', 1, 2, 3);
+
 my @newtable = pivot( table        => \@table, 
                       headings     => \@headings, 
                       fix_columns  => \@fix_cols, 
@@ -38,10 +34,10 @@ my @newtable = pivot( table        => \@table,
                       row_titles   => 0,
                       format       => '%5.2f'
                     );
-print "\nAfter:\n";
-out(\@headings, \@newtable);
-                      
+is_deeply( \@headings, \@expect_headings, 'headings ok' );
+is_deeply( \@newtable, \@expect_table,    'pivot ok' );
 
+###############################################################################
 @table = ( [ 'a', 'b', 'c', 1, 10, 1, 12 ],
            [ 'a', 'b', 'c', 2, 20, 2, 24 ],
            [ 'a', 'b', 'c', 3, 30, 3, 36 ],
@@ -54,8 +50,16 @@ out(\@headings, \@newtable);
 $pivot_col = 3;
 @sum_cols = ( 4, 5, 6 );
 
-print "\n\nBefore:\n";
-out(\@headings, \@table);
+@expect_table = (
+    ['a', 'b', 'c', 'V1',   '10.00', '20.00', '30.00', '60.00'],
+    ['',  '',  '',  'V 2',  ' 1.00', ' 2.00', ' 3.00', ' 6.00'],
+    ['',  '',  '',  'V  3', '12.00', '24.00', '36.00', '72.00'],
+    ['x', 'y', 'z', 'V1',   ' 1.00', ' 2.00', ' 3.00', ' 6.00'],
+    ['',  '',  '',  'V 2',  ' 9.00', ' 8.00', ' 7.00', '24.00'],
+    ['',  '',  '',  'V  3', '23.00', '34.00', '45.00', '102.00'],
+    );
+@expect_headings = ('A', 'B', 'C', '', '1', '2', '3', 'Sum');
+
 @newtable = pivot( table        => \@table, 
                       headings     => \@headings, 
                       fix_columns  => \@fix_cols, 
@@ -65,9 +69,10 @@ out(\@headings, \@table);
                       row_titles   => 1,
                       format       => '%5.2f'
                     );
-print "\nAfter:\n";
-out(\@headings, \@newtable);
+is_deeply( \@headings, \@expect_headings, 'headings ok' );
+is_deeply( \@newtable, \@expect_table,    'pivot ok' );
 
+###############################################################################
 @table = ( [ 'a', 'b', 'c', 1, 10000, 1, 12000 ],
            [ 'a', 'b', 'c', 2, 20, 2, 24000 ],
            [ 'a', 'b', 'c', 3, 30, 3, 36000 ],
@@ -76,6 +81,18 @@ out(\@headings, \@newtable);
            [ 'x', 'y', 'z', 3, 3, 7, 45000 ],
          );
 @headings = ( 'A', 'B', 'C', 'P', 'V1', 'V 2', 'V  3' );
+
+@expect_table = (
+    ['a', 'b', 'c', '  10.000,00', '      1,00', '  12.000,00', '     20,00', '      2,00', '  24.000,00', '     30,00', '      3,00', '  36.000,00'],
+    ['x', 'y', 'z', '      1,00', '      9,00', '  23.000,00', '      2,00', '      8,00', '  34.000,00', '      3,00', '      7,00', '  45.000,00'],
+    );
+@expect_headings = (
+    'A', 'B', 'C',
+    'V1 1', 'V 2 1', 'V  3 1',
+    'V1 2', 'V 2 2', 'V  3 2',
+    'V1 3', 'V 2 3', 'V  3 3',
+    );
+
 @newtable = pivot( table        => \@table, 
                       headings     => \@headings, 
                       fix_columns  => \@fix_cols, 
@@ -86,22 +103,34 @@ out(\@headings, \@newtable);
                       row_titles   => 1,
                       format       => \&format
                     );
-print "\nAfter (vertical):\n";
-out(\@headings, \@newtable);
+is_deeply( \@headings, \@expect_headings, 'headings ok' );
+is_deeply( \@newtable, \@expect_table,    'pivot ok' );
+
+###############################################################################
+# Test list concatenation for headers.
+@table = ( [ 'a', 'x', 1 ],
+           [ 'a', 'y', 2 ],
+           [ 'a', 'z', 3 ],
+         );
+@headings = ( 'A', 'B', 'C' );
+
+@expect_table = (
+    ['a', 1, 2, 3, 6],
+    );
+@expect_headings = ( 'A', 'x', 'y', 'z', 'Sum' );
+
+@newtable = pivot( table => \@table,
+                    headings => \@headings,
+                    pivot_column => 1,
+                    layout  => 'horizontal',
+                    row_sum => 'Sum',
+                 );
+
+is_deeply( \@headings, \@expect_headings, 'headings ok - list concat test' );
+is_deeply( \@newtable, \@expect_table,    'pivot ok - list concat test' );
 
 
-
-sub out {
-  my ($headings, $table) = @_;
-
-  print $_, "\t" foreach @$headings;
-  print "\n\n";
-  foreach (@$table) {
-    print $_, "\t" foreach @$_;
-    print "\n";
-  }
-}
-
+###############################################################################
 sub format {
   my $v = sprintf('%10.2f', $_[0]);
 
